/*******************************************************************************
Copyright (c) 1997-2004, Perforce Software, Inc.  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1.  Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.

2.  Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL PERFORCE SOFTWARE, INC. BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*******************************************************************************/

class ClientApi;
class PerlClientUser;

class PerlClientApi 
{
    public:
    		PerlClientApi();
		~PerlClientApi();

    SV *	Connect();
    SV *	Disconnect();
    SV *	Dropped();
    SV *	Run( const char *cmd, int argc, char * const *argv );

    void	SetClient( const char *c ) 	{ client->SetClient( c );    }
    void	SetCwd( const char *c )		{ client->SetCwd( c );	     }
    void	SetHost( const char *c )	{ client->SetHost( c );	     }
    void	SetLanguage( const char *c )	{ client->SetLanguage( c );  }
    void	SetPassword( const char *c )	{ client->SetPassword( c );  }
    void	SetPort( const char *c )	{ client->SetPort( c );	     }
    void	SetCharset( const char *c )	{ client->SetCharset( c );	     }
	void	SetTrans( int output, int content = -2,	int fnames = -2, int dialog = -2 ) {
		client->SetTrans( output, content, fnames, dialog );	     }
    void	SetUser( const char *c )	{ client->SetUser( c );      }
    void	SetProg( const char *c );

    void	SetInput( SV *i );

    SV *	GetClient();
    SV *	GetCwd();
    SV *	GetHost();
    SV *	GetLanguage();
    SV *	GetPassword();
    SV *	GetPort();
    SV *	GetCharset();
    SV *	GetUser();

    void	SetProtocol( const char *p, const char *v );
    StrPtr *	GetProtocol( const char *v );

    //
    // Handling command output
    //
    SV *	MergeErrors( int merge = -1 );
    SV *	GetFirstOutput();
    AV *	GetOutput();
    AV *	GetWarnings();
    AV *	GetErrors();

    I32		GetOutputCount();
    I32		GetWarningCount();
    I32		GetErrorCount();

    // Debugging support
    void	SetDebugLevel( int l );
    int		GetDebugLevel()			{ return debugLevel;	     }
    int		IsConnected()			{ return initCount;	     }

    //
    // Compatibility flags
    //
    enum 
    {
	CPT_MERGED	= 0x0001
    };

    private:
	ClientApi *		client;
	PerlClientUser *	ui;
	int			initCount;
	int			debugLevel;
	int			compatFlags;
};
